package org.example.auth

import io.ktor.http.HttpStatusCode
import org.example.dto.requests.SignInRequest
import org.example.dto.requests.SignUpRequest
import org.example.dto.responses.AuthResponse
import org.example.data.DataRepository
import io.ktor.server.application.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import org.example.dto.requests.ForgotPasswordRequest
import org.example.dto.requests.VerifyCodeRequest
import org.example.dto.responses.ForgotPasswordResponse
import org.example.dto.responses.PasswordResponse

fun Route.authRoutes() {
    post("/sign-up") {
        val request = call.receive<SignUpRequest>()
        println("/регистрация: $request")

        val user = DataRepository.createUser(request)
        println("пользователь создан: $user")

        val token = JwtConfig.makeToken(user.id)
        println("сгенерированный токен: $token")

        call.respond(AuthResponse(token = token))
    }

    post("/sign-in") {
        val request = call.receive<SignInRequest>()
        println("/вход: $request")

        val user = DataRepository.authenticate(request)
        if (user == null) {
            println("ошибка аутентификации для email: ${request.email}")
            throw IllegalArgumentException("Invalid credentials")
        }

        val token = JwtConfig.makeToken(user.id)
        println("успешный вход для: ${user.email}, токен: $token")

        call.respond(AuthResponse(token = token))
    }

    post("/forgot-password") {
        val request = call.receive<ForgotPasswordRequest>()
        println("запрос на сброс пароля для: ${request.email}")

        try {
            val code = DataRepository.initiatePasswordReset(request.email)
            println("код сброса пароля для ${request.email}: $code")

            call.respond(
                ForgotPasswordResponse(
                    message = "Код отправлен на email",
                    code = code
                )
            )
        } catch (e: IllegalArgumentException) {
            println("ошибка при сбросе пароля: ${e.message}")
            call.respond(
                HttpStatusCode.BadRequest,
                ForgotPasswordResponse(message = e.message ?: "Ошибка")
            )
        }
    }

    post("/verify-reset-code") {
        val request = call.receive<VerifyCodeRequest>()
        println("проверка кода ${request.code} для ${request.email}")

        try {
            val password = DataRepository.verifyResetCode(request.email, request.code)
            println("успешная проверка кода для ${request.email}, пароль: $password")

            call.respond(PasswordResponse(password = password))
        } catch (e: IllegalArgumentException) {
            println("неверный код для ${request.email}: ${request.code}")
            call.respond(
                HttpStatusCode.BadRequest,
                ForgotPasswordResponse(message = e.message ?: "Неверный код")
            )
        }
    }
}