package org.example.data

import org.example.data.Models.CartItem
import org.example.data.Models.Sneaker
import org.example.dto.requests.SignInRequest
import org.example.dto.requests.SignUpRequest
import org.example.data.Models.User
import org.example.data.Models.UserData
import java.util.concurrent.atomic.AtomicInteger

object DataRepository {
    private val users = mutableListOf<User>()
    private val idCounter = AtomicInteger(1)
    private val passwordResetCodes = mutableMapOf<String, String>()

    private val sneakers = mutableListOf(
        Sneaker(1, "Nike Air Max", 179.39, "Nike", "outdoor", "https://encrypted-tbn0.gstatic.com/images?q=tbn:ANd9GcSycZAb9c3gwov211CDjqS4n-RHt_4kAqDezw&s"),
        Sneaker(2, "Adidas Ultraboost", 169.39, "Adidas", "outdoor", "https://encrypted-tbn0.gstatic.com/images?q=tbn:ANd9GcSycZAb9c3gwov211CDjqS4n-RHt_4kAqDezw&s"),
        Sneaker(3, "Puma RS-X", 159.39, "Puma", "tennis", "https://encrypted-tbn0.gstatic.com/images?q=tbn:ANd9GcSycZAb9c3gwov211CDjqS4n-RHt_4kAqDezw&s")
    )

    private val userData = mutableMapOf<Int, UserData>()

    // ауф

    fun createUser(request: SignUpRequest): User {
        val user = User(
            id = idCounter.getAndIncrement(),
            email = request.email,
            password = request.password,
            name = request.name
        )
        users.add(user)
        return user
    }

    fun authenticate(request: SignInRequest): User? {
        return users.find { it.email == request.email && it.password == request.password }
    }

    fun findById(id: Int): User? {
        return users.find { it.id == id }
    }

    fun initiatePasswordReset(email: String): String {
        val user = users.find { it.email == email }
            ?: throw IllegalArgumentException("User not found")

        val code = (100000..999999).random().toString()
        passwordResetCodes[email] = code

        return code
    }

    fun verifyResetCode(email: String, code: String): String {
        val storedCode = passwordResetCodes[email]
            ?: throw IllegalArgumentException("No code requested for this email")

        if (storedCode != code) {
            throw IllegalArgumentException("Invalid code")
        }

        return users.find { it.email == email }?.password
            ?: throw IllegalArgumentException("User not found")
    }

    init {
        createUser(
            SignUpRequest(
                email = "test@example.com",
                password = "123456",
                name = "TestUser"
            )
        )
    }

    // кросовка

    fun getAllSneakers(): List<Sneaker> = sneakers.toList()

    fun getSneakerById(id: Int): Sneaker? = sneakers.find { it.id == id }

    fun addToFavorites(userId: Int, sneakerId: Int): Boolean {
        val sneaker = getSneakerById(sneakerId) ?: return false
        val data = userData.getOrPut(userId) { UserData(userId) }
        if (!data.favoriteSneakers.contains(sneakerId)) {
            data.favoriteSneakers.add(sneakerId)
            return true
        }
        return false
    }

    fun removeFromFavorites(userId: Int, sneakerId: Int): Boolean {
        return userData[userId]?.favoriteSneakers?.remove(sneakerId) ?: false
    }

    fun addToCart(userId: Int, sneakerId: Int, quantity: Int = 1): Boolean {
        val sneaker = getSneakerById(sneakerId) ?: return false
        val data = userData.getOrPut(userId) { UserData(userId) }
        val existingItem = data.cartItems.find { it.sneakerId == sneakerId }

        if (existingItem != null) {
            existingItem.quantity += quantity
        } else {
            data.cartItems.add(CartItem(sneakerId, quantity))
        }
        return true
    }

    fun removeFromCart(userId: Int, sneakerId: Int): Boolean {
        return userData[userId]?.cartItems?.removeIf { it.sneakerId == sneakerId } ?: false
    }

    fun getCart(userId: Int): List<CartItem> {
        return userData[userId]?.cartItems?.toList() ?: emptyList()
    }

    fun getFavorites(userId: Int): List<Int> {
        return userData[userId]?.favoriteSneakers?.toList() ?: emptyList()
    }
}